
//Sep 18th,2016 closed-loop LLC

#include "DSP2833x_Device.h"     // DSP2833x Headerfile Include File
#include "DSP2833x_Examples.h"   // DSP2833x Examples Include File

// Prototype statements for functions found within this file.
void InitEPwm1Example(void);
void InitEPwm2Example(void); 
void Setup_ADC(void);
//////////////////////////////////////////////////////////////

/*PWM*/
#define EPWM1_MIN_DB   30    //200ns
#define EPWM2_MIN_DB   30
#define period 		   750
//////////////////////////////////////////////////////////////

/*ADC*/
# define ADC_MODCLK  0x3 	// HSPCLK = SYSCLKOUT/2*ADC_MODCLK2 = 150/(2*3)   = 25.0 MHz
# define ADC_CKPS    0x1   	// ADC module clock = HSPCLK/2*ADC_CKPS   = 25.0MHz/(1*2) = 12.5MHz
# define ADC_SHCLK   0xf   	// S/H width in ADC module periods    = 16 ADC clocks

//////////////////////////////////////////////////////////////


/*PID*/
double 	ActualSpeed;
unsigned int  InputSpeed=period;                     //original speed
#define InputSpeed_min      period*0.5               //duty cycle 50%
#define InputSpeed_max      period*2                 //duty cycle (period-delay-dead)/period

#define SetSpeed            2248   				     //responding to 100kHz, Gain=30.367 
int     err=0, err_pre=0, e_err=0, e_err_pre=0;
int     Inc_Speed=0;    
   	
#define Kp       0.15
#define Ki       0.02
#define Kd       0.001
///////////////////////////////////////////////////////////////

void main(void)
{
  
// Step 1. Initialize System Control
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the DSP2833x_SysCtrl.c file.

   InitSysCtrl();


// Specific clock setting for ADC
// here ADC_MODCLK=3: HSPCLK = SYSCLKOUT/ADC_MODCLK/2;  HSPCLK=150M/6=25M
   EALLOW;
   SysCtrlRegs.HISPCP.all = ADC_MODCLK;	// HSPCLK = SYSCLKOUT/6
   EDIS;
/////////////////////////////////////////
        
  
// Step 2. Initalize GPIO:
// This example function is found in the DSP2833x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
// InitGpio();  // Skipped for this example

// For this case just init GPIO pins for ePWM1, ePWM2, ePWM3
// These functions are in the DSP2833x_EPwm.c file
   InitEPwm1Gpio();
   InitEPwm2Gpio();	

// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
   DINT;

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the DSP2833x_PieCtrl.c file.
   InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
   IER = 0x0000;
   IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in DSP2833x_DefaultIsr.c.
// This function is found in DSP2833x_PieVect.c.
   InitPieVectTable();

// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the F28335.cmd file.
//   MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
//   InitFlash();


   InitAdc();  // For this example, init the ADC
   Setup_ADC();






  

// Interrupts that are used in this example are re-mapped to
// ISR functions found within this file.
   EALLOW;  // This is needed to write to EALLOW protected registers
   EDIS;    // This is needed to disable write to EALLOW protected registers

// Step 4. Initialize all the Device Peripherals:
// This function is found in DSP2833x_InitPeripherals.c
// InitPeripherals();  // Not required for this example


//TBCLKSYNC is very important which is used to syn the PWM.
   EALLOW;
   SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
   EDIS;

   InitEPwm1Example();
   InitEPwm2Example();

   EALLOW;
   SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
   EDIS;
/////////////////////////////////////////////////


// Enable CPU INT3 which is connected to EPWM1-3 INT:
// IER |= M_INT3;

// Enable EPWM INTn in the PIE: Group 3 interrupt 1-3
// PieCtrlRegs.PIEIER3.bit.INTx1 = 1;
// PieCtrlRegs.PIEIER3.bit.INTx2 = 1;
// PieCtrlRegs.PIEIER3.bit.INTx3 = 1;

// Enable global Interrupts and higher priority real-time debug events:
// EINT;   // Enable Global interrupt INTM
   ERTM;   // Enable Global realtime interrupt DBGM



// Step 6. IDLE loop. Just sit and loop forever (optional):
   for(;;)
   {
     
       DELAY_US(200);                                    //samlping period is equal or larger than the pwm period*10
       AdcRegs.ADCTRL2.bit.RST_SEQ1 = 0x1;
	   AdcRegs.ADCTRL2.bit.SOC_SEQ1 = 0x1;  	 
	   while (AdcRegs.ADCST.bit.INT_SEQ1==0) {}          // Wait EOC
       AdcRegs.ADCST.bit.INT_SEQ1_CLR = 1;
	   ActualSpeed=(AdcMirror.ADCRESULT0+AdcMirror.ADCRESULT1)/2.0;

	   err_pre=err;                                      //refresh err_pre
	   err=SetSpeed-ActualSpeed;
	   e_err_pre=e_err;
	   e_err=err-err_pre;

       Inc_Speed=Kp*e_err+Ki*err+Kd*(e_err-e_err_pre);   //increment_mode
	   InputSpeed+=Inc_Speed;                            //position_mode
	    	
       if(InputSpeed<InputSpeed_min)
       		InputSpeed=InputSpeed_min;                   //Limit the output
	   if(InputSpeed>InputSpeed_max)
       		InputSpeed=InputSpeed_max;
     	         
	    EPwm1Regs.TBPRD = InputSpeed;					 //Refresh position
	    EPwm1Regs.CMPA.half.CMPA = InputSpeed/2;
	    EPwm2Regs.TBPRD = InputSpeed;
	    EPwm2Regs.CMPA.half.CMPA = InputSpeed/2;
   	    	
     }
}



void Setup_ADC(void)
{
   AdcRegs.ADCTRL3.bit.ADCCLKPS = ADC_CKPS;  //prescale HSPCLK
   AdcRegs.ADCTRL1.bit.CPS = 0;              //no prescale   	
   AdcRegs.ADCTRL1.bit.ACQ_PS = ADC_SHCLK;   //S/H window  

   AdcRegs.ADCTRL3.bit.SMODE_SEL= 0;         //SMODE_SEL: sequential sampling
   AdcRegs.ADCTRL1.bit.SEQ_CASC = 0;         //0 Non-Cascaded Mode; 1 Cascaded Mode



   AdcRegs.ADCMAXCONV.bit.MAX_CONV1 = 1; 	 // 1+1=2 conv's each sequencer (SEQ1)
   AdcRegs.ADCCHSELSEQ1.bit.CONV00 = 0x8;    //ADCINB0 :1000
   AdcRegs.ADCCHSELSEQ1.bit.CONV01 = 0x9;    //ADCINB1 :1001

   	
   AdcRegs.ADCTRL2.bit.RST_SEQ1 = 0x1;       //immediately reset sequencer to state CONV00
   AdcRegs.ADCTRL2.bit.INT_ENA_SEQ1 = 0x0;   //interrupt request by INT_SEQ1 is disabled
   AdcRegs.ADCTRL2.bit.INT_MOD_SEQ1 = 0x0;   //0: INT_SEQ1 is set at the end of every SEQ1 sequence.
/*
   AdcRegs.ADCMAXCONV.bit.MAX_CONV2 = 1; 	 // 1+1=2 conv's each sequencer (SEQ2)
   AdcRegs.ADCCHSELSEQ3.bit.CONV08 = 0x10;   //ADCINB2
   AdcRegs.ADCCHSELSEQ3.bit.CONV09 = 0x11;   //ADCINB3	
   AdcRegs.ADCTRL2.bit.RST_SEQ2 = 0x1;       //immediately reset sequencer to state CONV08
*/   	 
}




/////////////////////////////////////////////////////////////////////////////
void InitEPwm1Example()
{

   EPwm1Regs.TBPRD = period;                        // Set timer period   f=150*e6/(2*HSPCLKDIV*CLKDIV*TBPRD)
   EPwm1Regs.TBPHS.half.TBPHS = 0x0000;             // Phase is 0
   EPwm1Regs.TBCTR = 0x0000;                        // Clear counter

   // Setup TBCLK
   EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; 	// Count up
   EPwm1Regs.TBCTL.bit.PHSEN = TB_DISABLE;        	// Disable phase loading

   EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       	// Clock ratio to SYSCLKOUT   
   EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;          	// TBCLK=SYSCLKOUT/(HSPCLKDIV*CLKDIV)


   EPwm1Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    	//   SHDWAMODE 
   EPwm1Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
   EPwm1Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;  	//  Load on CTR = Zero: (TBCTR = 0x0000)
   EPwm1Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;  	//  Time-base counter equal to zero 
   // Setup compare
   EPwm1Regs.CMPA.half.CMPA = period/2;

   // Set actions
   EPwm1Regs.AQCTLA.bit.ZRO = AQ_CLEAR;             //when CMPA=TBCTR action. Set: PWM1A on 1; clear: pwm on 0
   EPwm1Regs.AQCTLA.bit.PRD = AQ_SET;


   //EPwm1Regs.AQCTLB.bit.ZRO = AQ_SET;          			
   //EPwm1Regs.AQCTLB.bit.PRD = AQ_CLEAR; 

   // Active Low PWMs - Setup Deadband
   EPwm1Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;      		//direct output or deadband
   EPwm1Regs.DBCTL.bit.POLSEL = DB_ACTV_LOC;            	//invert  or don't invert 
   EPwm1Regs.DBCTL.bit.IN_MODE = DBA_ALL;              		//expwm1A as the input source
   EPwm1Regs.DBRED = EPWM1_MIN_DB;                      	//Rising Edge Delay time
   EPwm1Regs.DBFED = EPWM1_MIN_DB;                      	//falling Edge Delay time

/*
   // Interrupt where we will change the Deadband
   EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;     	// Select INT on Zero event
   EPwm1Regs.ETSEL.bit.INTEN = 1;                	// Enable INT
   EPwm1Regs.ETPS.bit.INTPRD = ET_3RD;           	// Generate INT on 3rd event
*/

}


void InitEPwm2Example()
{

   EPwm2Regs.TBPRD =period;                       // Set timer period
   EPwm2Regs.TBPHS.half.TBPHS = 0;          	  // Phase is 0
   EPwm2Regs.TBCTR = 0x0000;                      // Clear counter

   // Setup TBCLK
   EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; // Count up
   EPwm2Regs.TBCTL.bit.PHSEN = TB_DISABLE;        // Disable phase loading
   EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       // Clock ratio to SYSCLKOUT
   EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;          // Slow just to observe on the scope

   EPwm2Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;    //   SHDWAMODE 
   EPwm2Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
   EPwm2Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;  //  Load on CTR = Zero: (TBCTR = 0x0000)
   EPwm2Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;  //  Time-base counter equal to zero 
   // Setup compare
   EPwm2Regs.CMPA.half.CMPA = period/2;

   // Set actions
   EPwm2Regs.AQCTLA.bit.ZRO = AQ_SET;              // Set PWM2A on Zero
   EPwm2Regs.AQCTLA.bit.PRD = AQ_CLEAR;


   //EPwm2Regs.AQCTLB.bit.ZRO = AQ_CLEAR;          // Set PWM2A on Zero
   //EPwm2Regs.AQCTLB.bit.PRD = AQ_SET;

   // Active Low complementary PWMs - setup the deadband
   EPwm2Regs.DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
   EPwm2Regs.DBCTL.bit.POLSEL = DB_ACTV_LOC;
   EPwm2Regs.DBCTL.bit.IN_MODE = DBA_ALL;
   EPwm2Regs.DBRED = EPWM2_MIN_DB;
   EPwm2Regs.DBFED = EPWM2_MIN_DB;

/*
   // Interrupt where we will modify the deadband
   EPwm2Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;      // Select INT on Zero event
   EPwm2Regs.ETSEL.bit.INTEN = 1;                 // Enable INT
   EPwm2Regs.ETPS.bit.INTPRD = ET_3RD;            // Generate INT on 3rd event
*/
}

//===========================================================================
// No more.
//===========================================================================

